////////////////////////////////////////////////////////////////////////////////




// ---------- rtvsD3dApp.cpp ----------

/*!

\file rtvsD3dApp.cpp
\brief Implementation of the rtvsD3dApp class
\author Gareth Edwards

*/




// ---------- include ----------

#include "../header/rtvsD3dApp.h"
#include <fstream>
#include <string>
#include <stack>

#define MAX_LENGTH 262144

// THINGS LIAM HAS ADDED!

// setup vars
int iter = 1;
float angle = 0;
float init_angle = 0;
float line_angle = 0;
char axiom[8];
char rule1[32];
char rule2[32];
char tree[MAX_LENGTH];
bool grow_tree;

// setup camera translation
float cam_x = 0, cam_y = -4.0f, cam_z = 10.0f;
void cam_right(float xdiff) { cam_x -= xdiff; }
void cam_up(float ydiff) { cam_y -= ydiff; }
void cam_zoom(float zdiff) { cam_z -= zdiff; }

// text boundary
RECT txt_bnd = { 50, 50, 550, 750 };

// functions

void getvars(const char* param_file, int &iter, float &angle, char* axiom, char* rule1, char* rule2)
{	
	std::ifstream in(param_file);
	int i;
	
	in >> iter;
	in >> angle;

	for(i = 0; i < 8; i++)
		axiom[i] = 0;
	in >> axiom;

	for(i = 0; i < 32; i++)
		rule1[i] = 0;
	in >> rule1;

	for(i = 0; i < 32; i++)
		rule2[i] = 0;
	in >> rule2;
	in.close();
}

// replaces test character with string
int rulecopy (char *rule, char test, char *from, char *to) {
	// keeps track of how many characters we've written
	int i = 0;
	// keep parsing until end of string
	while(*from)
	{
		// if test, copy rule
		if(*from == test) {
			int j;
			for(j = 0; rule[j]; j++) {
				*to = rule[j];
				to++;
			}
			i += j;
		}
		// otherwise keep the same character
		else {
			*to = *from;
			to++;
			i++;
		}
		from++;
		// ensure null termination
		*to = 0;
	}
	return i;
}

// copies a string
void copystring (char *from, char *to)
{
	while (*from)
	{
		*to = *from;
		to++; from++;
	}
	// ensure null termination
	*to = 0;
}

// generate the string for the new tree
void treegen(char *tree)
{
	// setup arrays, pointers, counter
	char temp2[MAX_LENGTH];
	char temp1[MAX_LENGTH];

	// decide method
	if(rule2[0] == 'n') {
		temp1[0] = 'F';
		temp1[1] = 0;
		for(int i = 0; i < iter; i++) {	
			rulecopy (rule1, 'F', temp1, temp2);
			// need to copy string back for iteration
			copystring(temp2, temp1);
		}
	}
	else {
		temp1[0] = 'X';
		temp1[1] = 0;
		for(int i = 0; i < iter; i++) {	
			rulecopy (rule1, 'X', temp1, temp2);
			rulecopy (rule2, 'F', temp2, temp1);
		}
	}

	// finally copy the tree for display
	copystring(temp2, tree);

	/* output string to file for testing
	std:: ofstream out("output.txt");
	out << tree;
	out.close();*/
}

// END OF THINGS LIAM HAS ADDED!


////////////////////////////////////////////////////////////////////////////////




// ---------- constructor ----------

/*!

\brief constructor
\author Gareth Edwards

\param int (id of this instance)


*/

rtvsD3dApp::rtvsD3dApp (int id)
{
	// initialise
    ZeroMemory( this, sizeof(rtvsD3dApp) );

	// store id
	_id = id;

	// key clicked
	currentKeyClicked = 1;
	
	getvars("1.txt", iter, angle, axiom, rule1, rule2);

	grow_tree = true;
}

////////////////////////////////////////////////////////////////////////////////




// ---------- framework ----------




// ---------- framework : cleanup ----------

/*!

\brief framework : cleanup
\author Gareth Edwards

\return bool (TRUE if ok)

*/

bool rtvsD3dApp::cleanup ()
{

	// ok
	return true;

}




// ---------- framework : cleanup dx ----------

/*!

\brief framework : cleanup dx
\author Gareth Edwards

\param LPDIRECT3DDEVICE9	(device)

\return bool (TRUE if ok)

*/

bool rtvsD3dApp::cleanupDX (LPDIRECT3DDEVICE9 pd3dDevice)
{

     // ---- invalidate the font object ----

    if( pFont != NULL )
    {
        int nNewRefCount = pFont->Release();

        if( nNewRefCount > 0 )
        {
            static char strError[256];
            sprintf_s ( strError, 256,
				"The font object failed to cleanup properly.\n"
                "Release() returned a reference count of %d",
				nNewRefCount );
            MessageBox( NULL, strError, "ERROR", MB_OK | MB_ICONEXCLAMATION );
        }

        pFont = NULL;
    }


    // ---- invalidate the vertex buffer object ----

    if( pVertexBuffer != NULL )
    {
        int nNewRefCount = pVertexBuffer->Release();

        if( nNewRefCount > 0 )
        {
            static char strError[256];
            sprintf_s ( strError, 256,
				"The vertex buffer object failed to cleanup properly.\n"
                "Release() returned a reference count of %d",
				nNewRefCount );
            MessageBox( NULL, strError, "ERROR", MB_OK | MB_ICONEXCLAMATION );
        }
        pVertexBuffer = NULL;
    }

	// ok
	return true;
}




// ---------- framework : display ----------

/*!

\brief framework : display
\author Gareth Edwards

\param LPDIRECT3DDEVICE9	(device)

\return bool (TRUE if ok)

*/

bool rtvsD3dApp::display (LPDIRECT3DDEVICE9 pd3dDevice)
{
 	// clear backbuffers
    pd3dDevice->Clear( 0,
		NULL,
		D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,
		D3DCOLOR_COLORVALUE(0.0f,0.0f,0.0f,0.0f),
		1.0f,
		0);

   // local matrices
    D3DXMATRIX matView;
    D3DXMATRIX matRotation;
    D3DXMATRIX matTranslation;
    D3DXMATRIX matWorld;

	// display flag
	bool lines = true;

	// view matrix
    D3DXMatrixIdentity( &matView );
    pd3dDevice->SetTransform( D3DTS_VIEW, &matView );

	// rotation matrix
	D3DXMatrixRotationYawPitchRoll	( &matRotation, D3DXToRadian(fSpinX), D3DXToRadian(fSpinY), 0 );

	// set render states
	pd3dDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

	// locate
	D3DXMatrixTranslation( &matTranslation, cam_x, cam_y, cam_z );
	matWorld = matRotation * matTranslation;
	pd3dDevice->SetTransform( D3DTS_WORLD, &matWorld );

	// THINGS LIAM HAS ADDED!

	if(grow_tree) {
		treegen(tree);
		grow_tree = false;
	}

	// draw
	if (lines)
	{	
		// set draw parameters
		pd3dDevice->SetTexture( 0, 0 );
		pd3dDevice->SetStreamSource( 0, pVertexBuffer, 0, sizeof(Vertex) );
		pd3dDevice->SetFVF( Vertex::FVF_Flags );
		pd3dDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
		pd3dDevice->SetMaterial( &lineMtrl );

		// update key clicked
		updateKeyboard();

		// setup text display
		char params[256];
		if(rule2[0] == 'n') {
		sprintf_s(params,
			"Position: (%f, %f)\n\nZoom: %f\n\nStep: %d\n\nInitial: %f\n\nAngle: %f\n\nAxiom: %s\n\nRule 1: F -> %s",
			cam_x, cam_y, cam_z, iter, init_angle, angle, axiom, rule1);
		pFont -> DrawText(0, params, -1, &txt_bnd, 0, fontCol);
		}
		else {
		sprintf_s(params,
			"Position: (%f, %f)\n\nZoom: %f\n\nStep: %d\n\nInitial: %f\n\nAngle: %f\n\nAxiom: %s\n\nRule 1: X -> %s\n\nRule 2: F -> %s",
			cam_x, cam_y, cam_z, iter, init_angle, angle,  axiom, rule2, rule1);
		pFont -> DrawText(0, params, -1, &txt_bnd, 0, fontCol);
		}

		// draw tree

		pd3dDevice->SetMaterial(&lineMtrl);

		std::stack<Vertex> cur_pos;
		std::stack<float> cur_angle;
		Vertex s, e, line;
		char *treepos;
		treepos = tree;

		s.x = 0;
		s.y = 0;
		s.z = 0;

		line.x = 0;
		line.y = 0.1f;
		line.z = 0;

		e.x = 0;
		e.y = 1;
		e.z = 0;

		line_angle = init_angle * 0.0174f;

		while (*treepos)
		{
			if (*treepos == 'F')
			{
				e.x = s.x + line.x * cosf(line_angle) - line.y * sinf(line_angle);
				e.y = s.y + line.x * sinf(line_angle) + line.y * cosf(line_angle);
				e.z = s.z + line.z;
				
				// update vertex buffer
				updateVertexBuffer(s, e);;

				// draw a single line
				pd3dDevice->DrawPrimitive( D3DPT_LINELIST, 0, 1 );

				s.x = e.x;
				s.y = e.y;
				s.z = e.z;
			}
         
			else if (*treepos == '[')
			{
				cur_pos.push(s);
				cur_angle.push(line_angle);
			}

			else if (*treepos == ']')
			{
				s = cur_pos.top();
				cur_pos.pop();
				line_angle = cur_angle.top();
				cur_angle.pop();
			}

			else if (*treepos == '+')
				line_angle += angle * 0.0174f;

			else if (*treepos == '-')
				line_angle -= angle * 0.0174f;

			treepos++;
      }

		// END OF THINGS LIAM HAS ADDED!
	}

	// ok
	return true;
}




// ---------- framework : setup ----------

/*!

\brief framework : setup
\author Gareth Edwards

\return bool (TRUE if ok)

*/

bool rtvsD3dApp::setup ()
{
	// setup a material for the trees
    ZeroMemory( &lineMtrl, sizeof(D3DMATERIAL9) );
	lineMtrl.Emissive.r = 0.3f;
	lineMtrl.Emissive.g = 0.2f;
	lineMtrl.Emissive.b = 0.0f;

	// ok
	return true;
}




// ---------- framework : setup dx ----------

/*!

\brief framework : setup dx
\author Gareth Edwards

\param LPDIRECT3DDEVICE9	(device)

\return bool (TRUE if ok)

*/

bool rtvsD3dApp::setupDX (LPDIRECT3DDEVICE9 pd3dDevice)
{
	// ---- turn lighting ON ----
	pd3dDevice->SetRenderState( D3DRS_LIGHTING , TRUE);

	// ---- ambient light ----
	pd3dDevice->SetRenderState( D3DRS_AMBIENT,D3DCOLOR_COLORVALUE( 0.0, 0.0, 0.0, 1.0));

	// ---- sun light ----
    pd3dDevice->SetLight( 0, &sunLight );
    pd3dDevice->LightEnable( 0, TRUE );

	// ---- back light ----
	pd3dDevice->SetLight( 1, &backLight );
    pd3dDevice->LightEnable( 1, TRUE );

	// ---- FONT ----
	fontCol = D3DCOLOR_COLORVALUE(1,1,1,1);
	D3DXCreateFont(
		pd3dDevice,
		30,								// height in pixels
		0,								// width in pixels (0 for default)
		400,							// thickness, 0-1000 OR FW_THIN (100), FW_NORMAL (400), FW_BOLD (700), FW_HEAVY (900)
		0,								// number of MipMaps to create. 0 creates a full chain - no scaling use 1
		false,							// 0/1 - true/false, do you want Italics
		DEFAULT_CHARSET,				// character Set - (Arabic, Greek, etc)
		OUT_DEFAULT_PRECIS,				// how precisely the output must match the font
		ANTIALIASED_QUALITY,			// ANTIALIASED_QUALITY, DEFAULT_QUALITY, DRAFT_QUALITY, and PROOF_QUALITY
		DEFAULT_PITCH | FF_DONTCARE,	// font pitch 
		"Arial",						// name of the required font or "" for system best match
		&pFont);

	// ---- LINE ----

	// ---- initialise vertex data ----
 	Vertex vertices[] =
	{
		//    x      y      z
		{  0.0f,  0.0f,  0.0f  },
		{  0.0f,  0.0f,  0.0f  },
	};

	// ---- create vertex buffer ----
	int numVertices = sizeof(vertices) / ( sizeof(float) * 3 );
	int numLines = numVertices / 2;
	pd3dDevice->CreateVertexBuffer( numVertices*sizeof(Vertex),
                                      D3DUSAGE_WRITEONLY,
                                      Vertex::FVF_Flags,
                                      //D3DPOOL_MANAGED, // does not have to be properly Released before calling IDirect3DDevice9::Reset
                                      D3DPOOL_DEFAULT,   // must be Released properly before calling IDirect3DDevice9::Reset
                                      &pVertexBuffer, NULL );

	// ---- block copy into vector vertex buffer ----
	void* pVertices = NULL;
	pVertexBuffer->Lock( 0, sizeof(vertices), (void**)&pVertices, 0 );
	memcpy( pVertices, vertices, sizeof(vertices) );
	pVertexBuffer->Unlock();

	// ok
	return true;
}




////////////////////////////////////////////////////////////////////////////////




// ---------- update vertex buffer ----------

/*!

\brief line - update vertex buffer
\author Gareth Edwards

\param 

\return bool (TRUE)

*/

DWORD rtvsD3dApp::updateVertexBuffer(Vertex s, Vertex e)
{

	// ---- update vector vertex buffer ----
	Vertex *pVertices = NULL;
	pVertexBuffer->Lock( 0, sizeof(Vertex) * 6, (void**)&pVertices, 0 );
	pVertices->x = (float)s.x;
	pVertices->y = (float)s.y;
	pVertices->z = (float)s.z;
	pVertices++;
	pVertices->x = (float)e.x;
	pVertices->y = (float)e.y;
	pVertices->z = (float)e.z;
	pVertexBuffer->Unlock();

	// ok
	return 1;
}



// ---------- update keyboard ----------

/*!

\brief update keyboard
\author Gareth Edwards

\param rtvsD3dSdk::stage* (pointer to RTVS Stage)

\return bool (TRUE if key updated)

*/

bool rtvsD3dApp::updateKeyboard ()
{
	// get key clicked

	// THINGS LIAM HAS ADDED!

	if(GetAsyncKeyState('1') & 0x8000f) {
		getvars("1.txt", iter, angle, axiom, rule1, rule2);
		treegen(tree);
	}
	else if(GetAsyncKeyState('2') & 0x8000f) {
		getvars("2.txt", iter, angle, axiom, rule1, rule2);
		treegen(tree);
	}
	else if(GetAsyncKeyState('3') & 0x8000f) {
		getvars("3.txt", iter, angle, axiom, rule1, rule2);
		treegen(tree);
	}
	else if(GetAsyncKeyState('4') & 0x8000f) {
		getvars("4.txt", iter, angle, axiom, rule1, rule2);
		treegen(tree);
	}
	else if(GetAsyncKeyState('5') & 0x8000f) {
		getvars("5.txt", iter, angle, axiom, rule1, rule2);
		treegen(tree);
	}
	else if(GetAsyncKeyState('6') & 0x8000f) {
		getvars("6.txt", iter, angle, axiom, rule1, rule2);
		treegen(tree);
	}
	else if(GetAsyncKeyState(VK_F1)) {
		iter = 1;
		treegen(tree);
	}
	else if(GetAsyncKeyState(VK_F2)) {
		iter = 2;
		treegen(tree);
	}
	else if(GetAsyncKeyState(VK_F3)) {
		iter = 3;
		treegen(tree);
	}
	else if(GetAsyncKeyState(VK_F4)) {
		iter = 4;
		treegen(tree);
	}
	else if(GetAsyncKeyState(VK_F5)) {
		iter = 5;
		treegen(tree);
	}
	else if(GetAsyncKeyState('Q') & 0x8000f)
		cam_zoom(-0.1f);
	else if(GetAsyncKeyState('E') & 0x8000f)
		cam_zoom(0.1f);
	else if(GetAsyncKeyState('W') & 0x8000f)
		cam_up(0.1f);
	else if(GetAsyncKeyState('A') & 0x8000f)
		cam_right(-0.1f);
	else if(GetAsyncKeyState('S') & 0x8000f)
		cam_up(-0.1f);
	else if(GetAsyncKeyState('D') & 0x8000f)
		cam_right(0.1f);
	else if(GetAsyncKeyState(VK_UP))
		init_angle -= 0.1f;
	else if(GetAsyncKeyState(VK_DOWN))
		init_angle += 0.1f;
	else if(GetAsyncKeyState(VK_LEFT))
		angle -= 0.1f;
	else if(GetAsyncKeyState(VK_RIGHT))
		angle += 0.1f;

	// END OF THINGS LIAM HAS ADDED

	return true;
}




////////////////////////////////////////////////////////////////////////////////




// ---------- getSpin ----------

/*!

\brief get x and y spin 
\author Gareth Edwards
\param float* ([out] spin around x axis)
\param float* ([out] spin around y axis)

*/

void rtvsD3dApp::getSpin (float* spinX, float* spinY)
{
	*spinX = fSpinX;
	*spinY = fSpinY;
}




// ---------- setSpin ----------

/*!

\brief set x and y spin 
\author Gareth Edwards
\param float (spin around x axis)
\param float (spin around y axis)

*/

void rtvsD3dApp::setSpin (float spinX, float spinY)
{
	fSpinX = spinX;
	fSpinY = spinY;
}




////////////////////////////////////////////////////////////////////////////////
